module flash_wr (
			clk,rst_n,led,
			flash_ce,flash_oe,flash_we,address,data
						);

	input              clk,rst_n     ;
	output             led           ;
	
	output             flash_ce      ;
	output             flash_oe      ;
	output             flash_we      ;
	//output             flash_ry_by   ;
	output  [20:0 ]    address       ;
	inout   [7 :0 ]    data          ;
	
	reg                led           ;
	reg                flash_ce      ;
	reg                flash_oe      ;
	reg                flash_we      ;
	reg     [20:0 ]    address       ;
	wire    [7 :0 ]    data          ;
	reg     [7 :0 ]    data_reg      ;
	reg     [7 :0 ]    data_reg_reg  ;
	reg     [20:0 ]    address_reg   ;
	reg     [7 :0 ]    data_in_reg   ;
	reg     [7 :0 ]    data_out_reg  ;
	reg     [9 :0 ]    state_cnt     ;
	
assign data = data_reg ;		

always @ ( posedge clk or negedge rst_n )
	if( !rst_n )
		begin
			flash_we <= 1 ;
			flash_ce <= 1 ;
			flash_oe <= 1 ;
			address <= 21'bz ;
			data_reg <= 8'bz ;
			data_reg_reg <= 8'b0 ;
			state_cnt <= 'd0 ;
			address_reg <= 21'b0 ;
			data_in_reg <= 8'b0 ;
			data_out_reg <= 8'b0 ;
			led <= 0 ;
		end
	else
		begin
			if( 10'b11_1111_1111 == state_cnt )
				begin
					if(21'b0_1111_1111_1111_1111_1111 <= address )
						state_cnt <= 'd0 ;
					else
						state_cnt <= 'd100 ;
				end
			else
				state_cnt <= state_cnt + 10'b00_0000_0001 ;
				
			case(state_cnt)
				//erase
				'd0:
					begin
						flash_we <= 1 ;
						flash_ce <= 1 ;
						flash_oe <= 1 ;
						address <= 21'b0_z ;
						data_reg <= 8'bz ;
					end
				'd1:
					begin
						flash_ce <= 0 ;
					end
				'd2:
					begin
						address <= 21'b0_0000_0000_1010_1010_1010 ;//AAA
						data_reg <= 8'b1010_1010 ;//AA
					end
				'd3:
					begin
						flash_we <= 0 ;
					end
				'd5:
					begin
						flash_we <= 1 ;
					end
				'd7:
					begin
						address <= 21'b0_0000_0000_0101_0101_0101 ;//555
						data_reg <= 8'b0101_0101 ;//55
					end
				'd8:
					begin
						flash_we <= 0 ;
					end
				'd10:
					begin
						flash_we <= 1 ;
					end
				'd12:
					begin
						address <= 21'b0_0000_0000_1010_1010_1010 ;//AAA
						data_reg <= 8'b1000_0000 ;//80
					end
				'd13:
					begin
						flash_we <= 0 ;
					end
				'd15:
					begin
						flash_we <= 1 ;
					end
				'd17:
					begin
						address <= 21'b0_0000_0000_1010_1010_1010 ;//AAA
						data_reg <= 8'b1010_1010 ;//AA
					end
				'd18:
					begin
						flash_we <= 0 ;
					end
				'd20:
					begin
						flash_we <= 1 ;
					end
				'd22:
					begin
						address <= 21'b0_0000_0000_0101_0101_0101 ;//555
						data_reg <= 8'b0101_0101 ;//55
					end
				'd23:
					begin
						flash_we <= 0 ;
					end
				'd25:
					begin
						flash_we <= 1 ;
					end
				'd27:
					begin
						address <= 21'b0_0000_0000_1010_1010_1010 ;//AAA
						data_reg <= 8'b0010_0000 ;//10
					end
				'd28:
					begin
						flash_we <= 0 ;
					end
				'd30:
					begin
						flash_we <= 1 ;
					end
				//write	
				'd100:
					begin
						address <= 21'b0_0000_0000_1010_1010_1010 ;//AAA
						data_reg <= 8'b1010_1010 ;//AA
					end
				'd101:
					begin
						flash_we <= 0 ;
					end
				'd103:
					begin
						flash_we <= 1 ;
					end
				'd105:
					begin
						address <= 21'b0_0000_0000_0101_0101_0101 ;//555
						data_reg <= 8'b0101_0101 ;//55
					end
				'd106:
					begin
						flash_we <= 0 ;
					end
				'd108:
					begin
						flash_we <= 1 ;
					end
				'd110:
					begin 
						address <= 21'b0_0000_0000_1010_1010_1010 ;//AAA
						data_reg <= 8'b1010_0000 ;//A0
					end
				'd111:
					begin
						flash_we <= 0 ;
					end
				'd113:
					begin
						flash_we <= 1 ;
						if( 21'b0_1111_1111_1111_1111_1111 <= address_reg )
							address_reg <= 21'b0_0000_0001_0000_0000_0000 ;
						else
							address_reg <= address_reg + 21'b0_0000_0000_0000_0000_0001 ;
					end
				'd115:
					begin
						//if( 21'b0_1111_1111_1111_1111_1111 <= address )
						//	address <= 21'b0_0000_0001_0000_0000_0000 ;
						//else
						//	address <= address + 21'b0_0000_0001_0000_0000_0000 ;//{$random}%10000 ;
						address <= address_reg ;
						data_reg <= data_reg_reg ;
						data_reg_reg <= data_reg_reg + 8'b0000_0001 ;//{$random}%1000 ;
					end
				'd116:
					begin
						flash_we <= 0 ;
						data_in_reg <= data_reg ;
					end
				'd118:
					begin
						flash_we <= 1 ;
					end
				//
				'd1000:
					begin
						flash_we <= 1 ;
					end
				'd1002:
					begin
						address <= address_reg ;
						data_reg <= 8'bz ;
					end
				'd1003:
					begin
						flash_oe <= 0 ;
					end
				'd1008:
					begin
						data_out_reg <= data ;
					end
				'd1009:
					begin
						flash_oe <= 1 ;
					end
				'd1010:
					begin
						if( data_out_reg==data_in_reg )
							led <= 0 ;
						else
							led <= 1 ;
					end
				/*'d996:
					begin
						data_reg <= 8'b1111_0000 ;//F0
					end
				'd997:
					begin
						flash_we <= 0 ;
					end
				'd999:
					begin
						flash_we <= 1 ;
					end*/
			endcase	
		end
endmodule			